package org.siqisource.agave.orm;

import java.util.Arrays;
import java.util.List;

import org.siqisource.agave.orm.expression.BetweenValueExpression;
import org.siqisource.agave.orm.expression.ListValueExpression;
import org.siqisource.agave.orm.expression.NoValueExpression;
import org.siqisource.agave.orm.expression.SingleValueExpression;
import org.siqisource.agave.orm.expression.SqlCompareExpression;
import org.siqisource.agave.utils.NameConverter;

public class SqlBuilder {

	public static final String AND_PT = " AND ( ";

	public static final String AND = " AND ";

	public static final String OR_PT = " OR ( ";

	public static final String OR = " OR ";

	public static final String END_PT = " ) ";


	public static SqlCompareExpression singleValue(String prefix, String propertyName, String compareSymbol, Object value,
			String suffix) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		return new SingleValueExpression(prefix, column, compareSymbol, value, suffix);
	}

	public static SqlCompareExpression noValue(String prefix, String propertyName, String compareSymbol, String suffix) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		return new NoValueExpression(prefix, column, compareSymbol, suffix);
	}

	public static SqlCompareExpression betweenValue(String prefix, String propertyName, Object begin, Object end, String suffix) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		return new BetweenValueExpression(prefix, column, begin, end, suffix);
	}

	public static SqlCompareExpression listValue(String prefix, String propertyName, String compareSymbol, List<?> value,
			String suffix) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		return new ListValueExpression(prefix, column, compareSymbol, value, suffix);
	}

	public static SqlCompareExpression listValue(String prefix, String propertyName, String compareSymbol, String values,
			String suffix) {
		String column = NameConverter.propertyToColumnWithTableAlias(propertyName);
		return new ListValueExpression(prefix, column, compareSymbol, Arrays.asList((Object[]) values.split(",")), suffix);
	}

}
